#include <QtNetwork>
#include "client.h"


Client::Client(QObject *parent) :
    Config(parent), networkSession(0)
{
    readConfig();
    nextConnexionIsAnImage = false;

    // if we did not find one, use IPv4 localhost
    if (getIpAddress().isEmpty())
        setIpAddress(QHostAddress(QHostAddress::LocalHost).toString());

    tcpSocket = new QTcpSocket(this);

    connect(tcpSocket, SIGNAL(readyRead()),
            this, SLOT(readNextItem()));
    connect(tcpSocket, SIGNAL(error(QAbstractSocket::SocketError)),
            this, SLOT(displayError(QAbstractSocket::SocketError)));
    connect(tcpSocket, SIGNAL(connected()),
            this, SLOT(startTransfer()));

    QNetworkConfigurationManager manager;
    if (manager.capabilities() & QNetworkConfigurationManager::NetworkSessionRequired) {
        // Get saved network configuration
        QSettings settings(QSettings::UserScope, QLatin1String("Trolltech"));
        settings.beginGroup(QLatin1String("QtNetwork"));
        const QString id = settings.value(QLatin1String("DefaultNetworkConfiguration")).toString();
        settings.endGroup();

        // If the saved network configuration is not currently discovered use the system default
        QNetworkConfiguration config = manager.configurationFromIdentifier(id);
        if ((config.state() & QNetworkConfiguration::Discovered) !=
            QNetworkConfiguration::Discovered) {
            config = manager.defaultConfiguration();
        }

        networkSession = new QNetworkSession(config, this);
        connect(networkSession, SIGNAL(opened()), this, SLOT(sessionOpened()));

        qDebug()<<"Opening network session.";
        networkSession->open();
    }
}

void Client::sendString(const QString &msg)
{
    qDebug() << "Sending message:" << msg;

    QDataStream out(&block, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_7);

    out << (quint16)0;
    out << msg;
    out.device()->seek(0);
    out << (quint16)(block.size() - sizeof(quint16));

    tcpSocket->abort();
    tcpSocket->connectToHost(getIpAddress(), getPort());
}

void Client::interpretCommandAndSendMessage(const QString &msg)
{
    QStringList msgList;
    qDebug() << "The message that will be sent: " << msg;

    // The parameters are compiled when requested (none by default)
    if (msg == "Patient - Scanner")
    {
        nextConnexionIsAnImage = true;
        qDebug() << "Next incoming message should be an image";
    }

    if (msg == "some command requiring parameters...") {
        msgList << "...";
    }

    // The message is sent to the server
    sendMessage(msg, msgList);
}

void Client::sendMessage(const QString &cmd, const QStringList &msgList)
{
    qDebug() << "Sending string list:";

    // Preparing the data stream
    QDataStream out(&block, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_7);
    out << (quint16)0;

    // Sending the 'Unique ID' as identifying header
    out << getUniqueId();
    qDebug() << getUniqueId();

    // Sending the 'Command'
    out << cmd;
    qDebug() << cmd;

    // Sending the number of parameters (if any!)
    int count = msgList.count();
    out << QString::number(count);
    qDebug() << QString::number(count);


    // Finally, sending all the parameters
    for (int i=0; i<count; i++) {
        QString msg = msgList[i];
        out << msg;
        qDebug() << msg;
    }

    // Tidying up before posting...
    out.device()->seek(0);
    out << (quint16)(block.size() - sizeof(quint16));

    // ... and sending!
    tcpSocket->abort();
    tcpSocket->connectToHost(getIpAddress(), getPort());
}


void Client::readString()
{
    qDebug()<< "Receiving message";

    QDataStream in(tcpSocket);
    in.setVersion(QDataStream::Qt_4_7);

    quint16 blockSize;
    qint64 bytes = tcpSocket->bytesAvailable();

    if (bytes < (int)sizeof(quint16))
        return;
    in >> blockSize;

    if (tcpSocket->bytesAvailable() < blockSize)
        return;

    QString newMessage;
    in >> newMessage;

    qDebug()<< "Received message:" << newMessage;
}

void Client::readNextItem()
{
    if (nextConnexionIsAnImage)
        readImage();
    else
        readStringList();
}

void Client::readStringList()
{
    qDebug()<< "Receiving StringList";

    QDataStream in(tcpSocket);
    in.setVersion(QDataStream::Qt_4_7);

    quint16 blockSize;
    qint64 bytes = tcpSocket->bytesAvailable();

    if (bytes < (int)sizeof(quint16))
        return;
    in >> blockSize;

    if (tcpSocket->bytesAvailable() < blockSize)
        return;

    // We first read the 'Unique ID'
    QString uniqueId;
    in >> uniqueId;
    qDebug() << "Message received from: " << uniqueId;

    // Then we read the command
    QString cmd;
    in >> cmd;
    qDebug() << "Command: " << cmd;

    // Then the number of parameters
    QString countString;
    bool isSuccess;
    in >> countString;
    qDebug() << countString << "parameters";
    int count = countString.toInt(&isSuccess);

    QStringList msgList;
    if (isSuccess) {
        QString msg;
        for (int i=0; i<count; i++)
        {
            in >> msg;
            msgList << msg;
            qDebug() << msg;
        }
    }

    if (cmd == "image") readImage();

    interpretAnswer(uniqueId, cmd, msgList);
}


void Client::readImage()
{
    nextConnexionIsAnImage = false;
    qDebug()<< "Receiving image";

    QBuffer* buffer = new QBuffer;
    buffer->open(QBuffer::ReadWrite);

    qint64 bytes = buffer->write(tcpSocket->readAll());
    buffer->seek(buffer->pos() - bytes);  // go back as many bytes as we just wrote so that it can be read

    //QImage image;			      // Construct a new QImage
    clientImage.loadFromData(buffer->buffer()); // Load the image from the receive buffer
    if (clientImage.isNull())		      // Check if the image was indeed received
            qDebug("The image is null. Something failed.");

    //=img.text("name");

    imageProvider.image = clientImage;
    emit imageAvailable();
}

void Client::displayError(QAbstractSocket::SocketError socketError)
{
    switch (socketError) {
    case QAbstractSocket::RemoteHostClosedError:
        break;
    case QAbstractSocket::HostNotFoundError:
        qDebug()<<"The host was not found. Please check the host name and port settings.";
        break;
    case QAbstractSocket::ConnectionRefusedError:
        qDebug()<<"The connection was refused by the peer. "
                                    << "Make sure the fortune server is running, "
                                    << "and check that the host name and port "
                                    << "settings are correct.";
        break;
    default:
        qDebug()<<"The following error occurred: "
                                    << tcpSocket->errorString();
    }
}

void Client::sessionOpened()
{
    // Save the used configuration
    QNetworkConfiguration config = networkSession->configuration();
    QString id;
    if (config.type() == QNetworkConfiguration::UserChoice)
        id = networkSession->sessionProperty(QLatin1String("UserChoiceConfiguration")).toString();
    else
        id = config.identifier();

    QSettings settings(QSettings::UserScope, QLatin1String("Trolltech"));
    settings.beginGroup(QLatin1String("QtNetwork"));
    settings.setValue(QLatin1String("DefaultNetworkConfiguration"), id);
    settings.endGroup();
}

void Client::startTransfer()
{
    tcpSocket->write(block);
}

void Client::setRootObject(QObject *object)
{
    rootObject = object;
}


void Client::addPatient(QString idNumber, QString title, QString firstName, QString lastName)
{
    if (rootObject == NULL) return;
    QVariant msg1 = (QVariant)idNumber;
    QVariant msg2 = (QVariant)title;
    QVariant msg3 = (QVariant)firstName;
    QVariant msg4 = (QVariant)lastName;
    QMetaObject::invokeMethod(rootObject, "addPatient",
                                Q_ARG(QVariant, msg1),
                                Q_ARG(QVariant, msg2),
                                Q_ARG(QVariant, msg3),
                                Q_ARG(QVariant, msg4) );
}

void Client::addDoctor(QString idNumber, QString title, QString firstName, QString lastName)
{
    if (rootObject == NULL) return;
    QVariant msg1 = (QVariant)idNumber;
    QVariant msg2 = (QVariant)title;
    QVariant msg3 = (QVariant)firstName;
    QVariant msg4 = (QVariant)lastName;
    QMetaObject::invokeMethod(rootObject, "addDoctor",
                                Q_ARG(QVariant, msg1),
                                Q_ARG(QVariant, msg2),
                                Q_ARG(QVariant, msg3),
                                Q_ARG(QVariant, msg4) );
}

void Client::clearPatients()
{
    if (rootObject == NULL) return;
    QMetaObject::invokeMethod(rootObject, "clearPatients");
}

void Client::clearDoctors()
{
    if (rootObject == NULL) return;
    QMetaObject::invokeMethod(rootObject, "clearDoctors");
}

// to finish!!
void Client::interpretAnswer(const QString uniqueId, const QString &cmd, const QStringList &msgList)
{
    if (cmd == "Patient - Select")
    {
        int itemCount = (msgList.count())/4;
        int itemIndex = 0;
        clearPatients();
        for (int i=0; i<itemCount; i++)
        {
            addPatient(msgList.at(itemIndex), msgList.at(itemIndex+1), msgList.at(itemIndex+2), msgList.at(itemIndex+3));
            itemIndex += 4;
        }
    }
    else if (cmd == "Doctor - Login")
    {
        int itemCount = (msgList.count())/4;
        int itemIndex = 0;
        clearDoctors();
        for (int i=0; i<itemCount; i++)
        {
            addDoctor(msgList.at(itemIndex), msgList.at(itemIndex+1), msgList.at(itemIndex+2), msgList.at(itemIndex+3));
            itemIndex += 4;
        }
    }
}

