#include <QtNetwork>
#include "server.h"

Server::Server(bool localMode, QObject *parent) :
    QObject(parent), tcpServer(0), networkSession(0)
{    
    // Taking into account the configuration options
    isLocalMode = localMode;    

    // The following is based on the "FortuneServer/FortuneClient" examples from the QtSDK examples
    QNetworkConfigurationManager manager;
    if (manager.capabilities() & QNetworkConfigurationManager::NetworkSessionRequired)
    {
        // Get saved network configuration
        QSettings settings(QSettings::UserScope, QLatin1String("Trolltech"));
        settings.beginGroup(QLatin1String("QtNetwork"));
        const QString id = settings.value(QLatin1String("DefaultNetworkConfiguration")).toString();
        settings.endGroup();

        // If the saved network configuration is not currently discovered use the system default
        QNetworkConfiguration config = manager.configurationFromIdentifier(id);
        if ((config.state() & QNetworkConfiguration::Discovered) !=
            QNetworkConfiguration::Discovered)
        {
            config = manager.defaultConfiguration();
        }

        networkSession = new QNetworkSession(config, this);
        connect(networkSession, SIGNAL(opened()), this, SLOT(sessionOpened()));

        qDebug("Opening a network session.");
        networkSession->open();
    }
    else
    {
        sessionOpened();
    }

    // Whenever a new connection comes in, it is managed by the following slot
    connect(tcpServer, SIGNAL(newConnection()),
            this, SLOT(acceptConnection()));
}


void Server::sessionOpened()
{
    // Save the used configuration
    if (networkSession) {
        QNetworkConfiguration config = networkSession->configuration();
        QString id;
        if (config.type() == QNetworkConfiguration::UserChoice)
            id = networkSession->sessionProperty(QLatin1String("UserChoiceConfiguration")).toString();
        else
            id = config.identifier();

        QSettings settings(QSettings::UserScope, QLatin1String("Trolltech"));
        settings.beginGroup(QLatin1String("QtNetwork"));
        settings.setValue(QLatin1String("DefaultNetworkConfiguration"), id);
        settings.endGroup();
    }

    tcpServer = new QTcpServer(this);
    if (!tcpServer->listen())
    {
        qDebug("Unable to start the server: " +(tcpServer->errorString()).toAscii());
        return;
    }
    QString ipAddress;
    QList<QHostAddress> ipAddressesList = QNetworkInterface::allAddresses();
    // use the first non-localhost IPv4 address
    for (int i = 0; i < ipAddressesList.size(); ++i)
    {
        if (ipAddressesList.at(i) != QHostAddress::LocalHost &&
            ipAddressesList.at(i).toIPv4Address())
        {
            ipAddress = ipAddressesList.at(i).toString();
            break;
        }
    }
    // if we did not find one, use IPv4 localhost
    if (ipAddress.isEmpty() || isLocalMode)
        ipAddress = QHostAddress(QHostAddress::LocalHost).toString();
    qDebug(tr("The server is running on \nIP: %1\nport: %2\n")
           .arg(ipAddress).arg(tcpServer->serverPort()).toAscii());
}

// This function is triggered when a new incoming connection is detected
void Server::acceptConnection()
{
    qDebug() << "Receiving a message";

    // Get the new socket from the incoming connection
    QTcpSocket* newSocket = tcpServer->nextPendingConnection();

    // Create a new server node for this socket
    // This object is used to handle the communication with the calling client
    ServerNode* newNode = new ServerNode(this, newSocket, &sql);

    connect(newSocket, SIGNAL(readyRead()),
            newNode, SLOT(readMessage()));
}

void Server::sendMessage(const QString &msg)
{
    QByteArray block;
    QDataStream out(&block, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_7);

    out << (quint16)0;
    out << msg;
    out.device()->seek(0);
    out << (quint16)(block.size() - sizeof(quint16));

    QTcpSocket *clientConnection = tcpServer->nextPendingConnection();
    connect(clientConnection, SIGNAL(disconnected()),
            clientConnection, SLOT(deleteLater()));

    clientConnection->write(block);
    clientConnection->disconnectFromHost();
}

void Server::prepare()
{
    qDebug() << "Preparing the database with initial dummy data";
    if (!sql.isOpen()) {
        qDebug() << "Error: the database is not open!";
        return;
    }

    qDebug() << "SQLite database opened successfuly\n";

    sql.createStructure();
    qDebug() << "Created SQLite database structure successfuly\n";

    sql.addDoctor("12345", "Dr.", "Haimso", "Cute");
    sql.addDoctor("13579", "Dr.", "Gregory", "House");
    sql.addDoctor("10001", "Dr.", "John", "Rambo");
    sql.addDoctor("10002", "Dr.", "John", "Rambo II");
    sql.addDoctor("10003", "Dr.", "John", "Rambo III");
    sql.addDoctor("10004", "Dr.", "John", "Rambo IV");

    qDebug() << "Added" << sql.getDoctorCount() << "doctors in the database\n";

    sql.addPatient("123456", "Mr.", "Edward", "Hyde");
    sql.addPatient("999999", "M.",  "Gaston", "Lagaffe");
    sql.addPatient("654321", "Mrs.","Adrienne", "Rambo");
    sql.addPatient("110100", "Mr.", "George", "Boole");
    sql.addPatient("124816", "Mr.", "Gordon", "Moore");
    sql.addPatient("246810", "M.",  "Jacques Victor Albert",  "De Broglie");
    sql.addPatient("123123", "Mme.","Yvonne", "Michu");
    qDebug() << "Added" << sql.getPatientCount() << "patients in the database\n";
}
